@@ -835,16 +835,16 @@ sub getLibTypes($$$$$$$$) {
         my $response = prompt("(y)es or (n)o", $default);
         
         if (uc($response) =~ /^(Y|YES)$/)  {
-            $staticlib_too = "y";
+            $staticlib_too = "Y";
         } elsif (uc($response) =~ /^(N|NO)$/)  {
-            $staticlib_too = "n";
+            $staticlib_too = "N";
         } else {
             print("'$response' isn't one of the choices.  \n" .
               "You must choose 'yes' or 'no' (or 'y' or 'n').\n");
             exit 12;
         }
     } else {
-        $staticlib_too = "n";
+        $staticlib_too = "N";
     }
     print("\n");
 
@@ -1048,6 +1048,19 @@ sub getSse($) {
 }
 
 
+sub getIcon($$) {
+
+    my ($platform, $wantIconR) = @_;
+
+    if ($platform eq 'WINDOWS') {
+        print("Include an icon in each executable?\n");
+        $$wantIconR = promptYesNo("y");
+    } else {
+        $$wantIconR = $FALSE;
+    }
+}
+
+
 
 # TODO: These should do test compiles to see if the headers are in the
 # default search path, both to create a default to offer and to issue a
@@ -2082,6 +2095,8 @@ getSse(\my $wantSse);
 
 findProcessManagement(\my $dontHaveProcessMgmt);
 
+getIcon($platform, \my $wantIcon);
+
 #******************************************************************************
 #
 #  FIND THE PREREQUISITE LIBRARIES
@@ -2370,6 +2385,9 @@ if ($platform eq "GNU") {
     if ($subplatform ne "cygwin") {
         push(@config_mk, "MSVCRT = Y\n");
     }
+    if ($wantIcon) {
+        push(@config_mk, 'WINICON_OBJECT = $(BUILDDIR)/icon.netpbm.o');
+    }
 } elsif ($platform eq "BEOS") {
     push(@config_mk, "LDSHLIB = -nostart\n");
 } elsif ($platform eq "OPENBSD") {
