@@ -19,7 +19,7 @@
    describe the Lempel-Ziv base.
 */
 
-
+#define _BSD_SOURCE   /* for strcaseeq */
 #include <string.h>
 #include <assert.h>
 
@@ -1218,6 +1218,7 @@ lzwReadByte(struct decompressor * const decompP,
                         *endOfImageP = TRUE;
                         *errorP = NULL;
                     } else {
+                        *endOfImageP = FALSE;
                         expandCodeOntoStack(decompP, code, errorP);
                         if (!*errorP)
                             *dataReadP = popStack(&decompP->stack);
@@ -1302,7 +1303,7 @@ renderRow(unsigned char *    const cmapIndexRow,
   Convert one row of cmap indexes to PPM/PGM/PBM output.
 
   Render the alpha row to *alphaFileP iff 'alphabits' is non-NULL.  If
-  'haveTransColor' is false, render all zeroes (i.e. the row is
+  'haveTransColor' is false, render all white (i.e. the row is
   opaque).  'alphabits' is otherwise just a one-row buffer for us to use
   in rendering the alpha row.
   
@@ -1423,8 +1424,6 @@ makePnmRow(struct decompressor * const decompP,
 
             lzwReadByte(decompP, &readColorIndex, &endOfImage, &readError);
 
-            assert(*errorP == NULL);
-
             verifyPixelRead(endOfImage, readError, cols, rows, errorP);
 
             if (readError)
