@@ -114,14 +114,16 @@
  * $Id$
  */
 
-/******************************************************************************\
+/*****************************************************************************\
 * Includes.
-\******************************************************************************/
+\*****************************************************************************/
 
 #include <stdio.h>
 #include <stdlib.h>
 #include <assert.h>
 
+#include "pm.h"
+
 #include "jasper/jas_types.h"
 #include "jasper/jas_math.h"
 #include "jasper/jas_tvp.h"
@@ -136,9 +138,9 @@
 #include "jpc_t1dec.h"
 #include "jpc_math.h"
 
-/******************************************************************************\
+/*****************************************************************************\
 *
-\******************************************************************************/
+\*****************************************************************************/
 
 #define JPC_MHSOC   0x0001
   /* In the main header, expecting a SOC marker segment. */
@@ -275,9 +277,9 @@ jpc_dec_mstabent_t jpc_dec_mstab[] = {
     {0, JPC_MH | JPC_TPH, jpc_dec_process_unk}
 };
 
-/******************************************************************************\
+/*****************************************************************************\
 * The main entry point for the JPEG-2000 decoder.
-\******************************************************************************/
+\*****************************************************************************/
 
 jas_image_t *jpc_decode(jas_stream_t *in, char *optstr)
 {
