@@ -5,7 +5,7 @@
 
 typedef struct {
     short           magic;
-    char            storage;
+    unsigned char   storage;
     char            bpc;            /* pixel size: 1 = bytes, 2 = shorts */
     unsigned short  dimension;      /* 1 = single row, 2 = B/W, 3 = RGB */
     unsigned short  xsize,          /* width in pixels */
