@@ -162,7 +162,7 @@ putinit(struct cmdlineInfo const cmdline) {
     /* Set raster graphics resolution */
     printf("\033*t%dR", cmdline.dpi);
 
-    /* Start raster graphics, relative adressing */
+    /* Start raster graphics, relative addressing */
     printf("\033*r1A");
 
     bitsperitem = 1;
@@ -545,7 +545,7 @@ main(int argc, char * argv[]) {
 
     struct cmdlineInfo cmdline;
     FILE * ifP;
-    bool eof;
+    int eof;
 
     pbm_init(&argc, argv);
 
