@@ -13,6 +13,7 @@
    Contributed to the public domain.
 =============================================================================*/
 #define _XOPEN_SOURCE
+#define _BSD_SOURCE  /* Make SIGWINCH defined on OpenBSD */
 
 #include <stdarg.h>
 #include <unistd.h>
@@ -133,7 +134,7 @@ createPipeFeeder(void          pipeFeederRtn(int, void *),
     int pipeToFeed[2];
     pid_t rc;
 
-    pipe(pipeToFeed);
+    pm_pipe(pipeToFeed);
     rc = fork();
     if (rc < 0) {
         pm_error("fork() of stdin feeder failed.  errno=%d (%s)", 
@@ -179,7 +180,7 @@ spawnProcessor(const char *  const progName,
     pid_t rc;
 
     if (pipeStdout)
-        pipe(stdoutpipe);
+        pm_pipe(stdoutpipe);
 
     rc = fork();
     if (rc < 0) {
@@ -220,77 +221,88 @@ spawnProcessor(const char *  const progName,
 static const char *
 signalName(unsigned int const signalClass) {
 
-    if (signalClass <= SIGSYS) {
-        switch (signalClass) {
-        case SIGHUP:
-            return "SIGHUP";
-        case SIGINT:
-            return "SIGINT";
-        case SIGQUIT:
-            return "SIGQUIT";
-        case SIGILL:
-            return "SIGILL";
-        case SIGTRAP:
-            return "SIGTRAP";
-        case SIGABRT:
-            return "SIGABRT";
-        case SIGBUS:
-            return "SIGBUS";
-        case SIGFPE:
-            return "SIGFPE";
-        case SIGKILL:
-            return "SIGKILL";
-        case SIGUSR1:
-            return "SIGUSR1";
-        case SIGSEGV:
-            return "SIGSEGV";
-        case SIGUSR2:
-            return "SIGUSR2";
-        case SIGPIPE:
-            return "SIGPIPE";
-        case SIGALRM:
-            return "SIGALRM";
-        case SIGTERM:
-            return "SIGTERM";
-        case SIGCHLD:
-            return "SIGCHLD";
-        case SIGCONT:
-            return "SIGCONT";
-        case SIGSTOP:
-            return "SIGSTOP";
-        case SIGTSTP:
-            return "SIGTSTP";
-        case SIGTTIN:
-            return "SIGTTIN";
-        case SIGTTOU:
-            return "SIGTTOU";
-        case SIGURG:
-            return "SIGURG";
-        case SIGXCPU:
-            return "SIGXCPU";
-        case SIGXFSZ:
-            return "SIGXFSZ";
-        case SIGVTALRM:
-            return "SIGVTALRM";
-        case SIGPROF:
-            return "SIGPROF";
-        case SIGWINCH:
-            return "SIGWINCH";
-        case SIGIO:
-            return "SIGIO";
-        case SIGSYS:
-            return "SIGSYS";
-        default:
-            return "???";
+/* There are various signal classes that are not universally defined,
+   so we make a half-hearted attempt to determine whether they are and
+   not try to recognize the ones that aren't.  We do this by testing
+   whether a macro is defind with the signal class name.  That could give
+   a false negative, because the signal class name isn't necessarily
+   defined as a macro, but it's a really, really small problem to miss
+   one of these signal classes here, so we don't bother with all the work
+   it would take to do it right.
 
-            /* There are various other signal classes on some systems, but
-               not defined by POSIX, including: SIGPWR, SIGLOST, SIGINFO
-            */
-        }
-    } else if ((int)signalClass >= SIGRTMIN && (int)signalClass <= SIGRTMAX)
-        return "SIGRTxxx";
-    else
+   OpenBSD does not have SIGWINCH and SIGIO in 2013.  Everyone else seems
+   to have it.
+*/
+    switch (signalClass) {
+    case SIGHUP: /* POSIX.1 */
+        return "SIGHUP";
+    case SIGINT: /* POSIX.1 */
+        return "SIGINT";
+    case SIGQUIT: /* POSIX.1 */
+        return "SIGQUIT";
+    case SIGILL: /* POSIX.1 */
+        return "SIGILL";
+    case SIGTRAP:
+        return "SIGTRAP";
+    case SIGABRT: /* POSIX.1 */
+        return "SIGABRT";
+    case SIGBUS:
+        return "SIGBUS";
+    case SIGFPE: /* POSIX.1 */
+        return "SIGFPE";
+    case SIGKILL: /* POSIX.1 */
+        return "SIGKILL";
+    case SIGUSR1: /* POSIX.1 */
+        return "SIGUSR1";
+    case SIGSEGV: /* POSIX.1 */
+        return "SIGSEGV";
+    case SIGUSR2: /* POSIX.1 */
+        return "SIGUSR2";
+    case SIGPIPE: /* POSIX.1 */
+        return "SIGPIPE";
+    case SIGALRM: /* POSIX.1 */
+        return "SIGALRM";
+    case SIGTERM: /* POSIX.1 */
+        return "SIGTERM";
+    case SIGCHLD: /* POSIX.1 */
+        return "SIGCHLD";
+    case SIGCONT: /* POSIX.1 */
+        return "SIGCONT";
+    case SIGSTOP: /* POSIX.1 */
+        return "SIGSTOP";
+    case SIGTSTP: /* POSIX.1 */
+        return "SIGTSTP";
+    case SIGTTIN: /* POSIX.1 */
+        return "SIGTTIN";
+    case SIGTTOU: /* POSIX.1 */
+        return "SIGTTOU";
+    case SIGURG:
+        return "SIGURG";
+    case SIGXCPU:
+        return "SIGXCPU";
+    case SIGXFSZ:
+        return "SIGXFSZ";
+    case SIGVTALRM:
+        return "SIGVTALRM";
+    case SIGPROF:
+        return "SIGPROF";
+#ifdef SIGWINCH
+    case SIGWINCH:
+        return "SIGWINCH";
+#endif
+#ifdef SIGIO
+    case SIGIO:
+        return "SIGIO";
+#endif
+#ifdef SIGPWR
+    case SIGPWR:
+        return "SIGPWR";
+#endif
+    case SIGSYS:
+        return "SIGSYS";
+    default:
         return "???";
+    }
 }
 
 
@@ -419,7 +431,7 @@ pm_system_vp(const char *    const progName,
         /* Run a child process for the program that sends its Standard Output
            to our Standard Output
         */
-        spawnProcessor(progName, argArray, STDIN, NULL, &processorPid);
+        spawnProcessor(progName, argArray, progStdinFd, NULL, &processorPid);
     }
 
     cleanupProcessorProcess(processorPid);
@@ -504,20 +516,20 @@ pm_feed_from_memory(int    const pipeToFeedFd,
 
     struct bufferDesc * const inputBufferP = feederParm;
     
-    FILE * const outfile = fdopen(pipeToFeedFd, "w");
+    FILE * const outFileP = fdopen(pipeToFeedFd, "w");
     
-    int bytesTransferred;
+    size_t bytesTransferred;
 
     /* The following signals (and normally kills) the process with
        SIGPIPE if the pipe does not take all 'size' bytes.
     */
     bytesTransferred = 
-        fwrite(inputBufferP->buffer, 1, inputBufferP->size, outfile);
+        fwrite(inputBufferP->buffer, 1, inputBufferP->size, outFileP);
 
     if (inputBufferP->bytesTransferredP)
         *(inputBufferP->bytesTransferredP) = bytesTransferred;
 
-    fclose(outfile);
+    fclose(outFileP);
 }
 
 
@@ -528,14 +540,14 @@ pm_accept_to_memory(int             const pipetosuckFd,
 
     struct bufferDesc * const outputBufferP = accepterParm;
     
-    FILE * const infile = fdopen(pipetosuckFd, "r");
+    FILE * const inFileP = fdopen(pipetosuckFd, "r");
 
-    int bytesTransferred;
+    size_t bytesTransferred;
 
     bytesTransferred =
-        fread(outputBufferP->buffer, 1, outputBufferP->size, infile);
+        fread(outputBufferP->buffer, 1, outputBufferP->size, inFileP);
 
-    fclose(infile);
+    fclose(inFileP);
 
     if (outputBufferP->bytesTransferredP)
         *(outputBufferP->bytesTransferredP) = bytesTransferred;
